/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://java.net/hg/bluebill-mobile~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.taxonomy.factsheet.ui;

import javax.annotation.Nonnull;
import it.tidalwave.util.ui.UserNotificationWithFeedback;
import it.tidalwave.util.ui.LockableView;
import it.tidalwave.role.ui.PresentationModel;
import it.tidalwave.bluebill.taxonomy.mobile.Taxon;
import it.tidalwave.util.ui.UserNotification;

/***********************************************************************************************************************
 *
 * @stereotype View
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public interface TaxonFactSheetView extends LockableView
  {
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void renderTaxon (@Nonnull Taxon taxon); // FIXME: must use a PresentationModel
    
    /*******************************************************************************************************************
     *
     * Renders the data.
     *
     ******************************************************************************************************************/
    public void populate (@Nonnull PresentationModel presentationModel);
    
    /*******************************************************************************************************************
     *
     * Shows a notification that there's no data for this taxon.
     * 
     * @param  notification  the notification message
     *
     ******************************************************************************************************************/
    public void notifyNoData (@Nonnull UserNotification notification);
    
    /*******************************************************************************************************************
     *
     * Shows a confirmation that an operating was completed with success.
     * 
     * @param  notification  the notification message
     *
     ******************************************************************************************************************/
    public void notifyCompletion (@Nonnull UserNotification notification);
    
    /*******************************************************************************************************************
     *
     * Shows the notification that an operating caused an error.
     * 
     * @param  notification  the notification message
     *
     ******************************************************************************************************************/
    public void notifyError (@Nonnull UserNotification notification);
    
    /*******************************************************************************************************************
     *
     * Notifies that the SD card is not ready.
     * 
     * @param notification   the notification
     *
     ******************************************************************************************************************/
    public void notifySdCardNotReady (@Nonnull UserNotificationWithFeedback notification);

    /*******************************************************************************************************************
     *
     * Asks for confirmation to delete a media item.
     * 
     * @param notifcation   the notification
     *
     ******************************************************************************************************************/
    public void confirmToDeleteMedia (@Nonnull UserNotificationWithFeedback notifcation);

    /*******************************************************************************************************************
     *
     * Asks for confirmation to download a media item.
     * 
     * @param notification   the notification
     *
     ******************************************************************************************************************/
    public void confirmToDownloadMedia (@Nonnull UserNotificationWithFeedback notification);

    /*******************************************************************************************************************
     *
     * Opens a web page (typically related to a media item).
     * 
     * @param url  the URL
     *
     ******************************************************************************************************************/
    public void openWebPage (@Nonnull String url);
  }
